@@ -1,8 +1,8 @@
 """
-compare.py
-
 Functions for comparing Python values, and for rendering the results of
 those comparisons as HTML.
+
+compare.py
 """
 
 import re
@@ -61,8 +61,8 @@ def strict_equality_checker(val, ref):
                 )
             else:
                 diff = "Actual:<br>\n{}<br><br>\nExpected:<br>\n{}".format(
-                    repr(val),
-                    repr(ref)
+                    html_tools.dynamic_html_repr(val),
+                    html_tools.dynamic_html_repr(ref)
                 )
 
         return {
@@ -151,6 +151,7 @@ The absolute tolerance for floating-point similarity (see
 `cmath.isclose`).
 """
 
+
 def numbers_are_equalish(val, ref):
     """
     Uses `cmath.isclose` to compare two floating-point numbers, and
