@@ -1,8 +1,10 @@
 """
+The `Context` class and related functions which form the backbone of the
+testing process.
+
 contexts.py
 
-The `Context` class and related functions which form the backbone of the
-testing process. A `Context` represents a specific situation derived from
+A `Context` represents a specific situation derived from
 other contexts and ultimately submitted and/or solution code.
 `potluck.rubrics.Goal` objects actually evaluate things, but they depend
 on one or more contexts to provide material to be evaluated.
@@ -91,20 +93,20 @@ def show_all_context_values(context):
 
 class Context:
     """
-    A Context represents some kind of product of submitted code, like an
+    Represents some kind of product of submitted code, like an
     AST tree, or produced output. Contexts can specialize each other, so
     for example a regular expression could be used to transform a context
     containing all output from a program into a context containing just a
     single line of output. Context objects organize context_builder
     functions into hierarchies, and manage caching of their results.
 
-    A Context object also needs to know how to render its results into
+    A `Context` object also needs to know how to render its results into
     HTML to display to the user as part of the feedback process.
 
-    A Context object, via its context_builder function, knows how to
+    A `Context` object, via its context_builder function, knows how to
     produce, on demand, a dictionary mapping context keys to values.
     The base context (and therefore all derived contexts) will always
-    include the following slots:
+    include the following `potluck.context_utils.BASE_CONTEXT_SLOTS`:
 
         - "task_info": A task info dictionary for the current task, which
             will have a "specification" key containing the specification
@@ -275,8 +277,8 @@ class Context:
         changes in our dependencies.
 
         A base context is required, and should be a dictionary; normally
-        it should have "task_info", "username", "submission_root", and
-        "default_file" slots already populated.
+        it should have all the `potluck.context_utils.BASE_CONTEXT_SLOTS`
+        already populated.
         """
         rerun = False
         if not self.cache_values:
@@ -323,7 +325,7 @@ class Context:
                 elif self.failure_explanation:
                     msg = self.failure_explanation(prev_context, e)
                 else:
-                    msg = "Context builder crashed."
+                    msg = "Test setup failed."
                 raise context_utils.ContextCreationError(self, msg, e)
             self.cache_timestamp = time.time()
             # Return shallow copy of cached value
@@ -796,6 +798,8 @@ class FileContext(AutoContext):
     target directory for multi-file submissions. If no target file is
     identified, the default_file context slot value is used to target
     the submission's default file.
+
+    Establishes 'ref_*' slots for each normal slot it establishes.
     """
     def __init__(self, target_file=None, depends=None, hidden=True):
         """
@@ -826,8 +830,12 @@ class FileContext(AutoContext):
             """
             Establishes 'filename' and 'file_path' slots based on the
             target_file value given to the `FileContext` that this is
-            the context builder function for.
+            the context builder function for. Also establishes
+            'ref_filename' and 'ref_file_path' slots pointing to the
+            equivalent file in the solution code.
             """
+            task_info = context_utils.extract(prev_context, "task_info")
+            soln_root = task_info["specification"].soln_path
             submission_root = context_utils.extract(
                 prev_context,
                 "submission_root"
@@ -854,9 +862,15 @@ class FileContext(AutoContext):
                     submission_root,
                     actual_filename
                 )
+                # Even if we're grading a custom-named file, the solution
+                # won't have a custom name...
+                ref_target = os.path.join(soln_root, default_filename)
                 target_filename = default_filename
             else:
+                # TODO: How can we properly handle ref targets for
+                # custom-named files here?
                 full_target = os.path.join(submission_root, file_target)
+                ref_target = os.path.join(soln_root, file_target)
                 _, target_filename = os.path.split(full_target)
             if not os.path.exists(submission_root):
                 # If the submission root directory is missing...
@@ -876,11 +890,19 @@ class FileContext(AutoContext):
                         f" '{full_target}' does not exist."
                     )
                 )
+            elif not os.path.exists(ref_target):
+                # If there is no equivalent solution file
+                raise context_utils.ContextCreationError(
+                    self,
+                    f"No solution file '{ref_target}' is available."
+                )
             # else both submission root and full target exist
 
             return {
                 "filename": target_filename,
-                "file_path": full_target
+                "file_path": full_target,
+                "ref_filename": target_filename,
+                "ref_file_path": ref_target
             }
 
         # Now we can call the super-class init with the context builder
@@ -900,12 +922,20 @@ class FileContext(AutoContext):
 
         # Finally, we can register ourselves as an auto-context for the
         # "filename" and "file_path" slots.
-        self.register("filename", "file_path")
+        self.register(
+            "filename",
+            "file_path",
+            "ref_filename",
+            "ref_file_path"
+        )
 
 
 # Register a factory for a default FileContext as an on-demand option
 # for "filename" and "file_path" slots:
-AutoContext.on_demand((lambda: FileContext()), "filename", "file_path")
+AutoContext.on_demand(
+    (lambda: FileContext()),
+    "filename", "file_path", "ref_filename", "ref_file_path"
+)
 
 
 class CodeContext(AutoContext):
@@ -962,7 +992,7 @@ class CodeContext(AutoContext):
             except Exception as e:
                 raise context_utils.ContextCreationError(
                     self,
-                    "Unable to parse submitted file '{filename}'.",
+                    f"Unable to parse submitted file '{filename}'.",
                     cause=e
                 )
 
@@ -1025,10 +1055,11 @@ AutoContext.on_demand(
 
 class SolnCodeContext(AutoContext):
     """
-    Requires "filename" and "file_path" slots (see `FileContext`), and
-    establishes a "ref_source" slot which contains the raw text of the
-    equivalent file from the solution code, along with a "ref_scope" slot
-    which contains the parsed AST from the solution code.
+    Requires "ref_filename" and "ref_file_path" slots (see
+    `FileContext`), and establishes a "ref_source" slot which contains
+    the raw text of the equivalent file from the solution code, along
+    with a "ref_scope" slot which contains the parsed AST from the
+    solution code.
 
     "task_info" and "submission_root" slots are also required, but those
     should always be present.
@@ -1048,7 +1079,7 @@ class SolnCodeContext(AutoContext):
         def establish_context(prev_context):
             """
             Establishes the following context slots based on the
-            "file_path" slot, by reading the solution version of the
+            "ref_file_path" slot, by reading the solution version of the
             indicated file:
 
                 ref_source: The source of the solution file.
@@ -1056,26 +1087,19 @@ class SolnCodeContext(AutoContext):
                     solution file.
                 ref_top_scope: As above, but won't be modified.
             """
-            target = context_utils.extract(prev_context, "file_path")
-            sr = context_utils.extract(prev_context, "submission_root")
-            task_info = context_utils.extract(prev_context, "task_info")
-            if not target.startswith(sr):
-                raise context_utils.ContextCreationError(
-                    self,
-                    "Target file isn't inside submission root!"
-                )
-            in_submission = target[len(sr):]
-            if in_submission.startswith(os.path.sep):
-                in_submission = in_submission[len(os.path.sep):]
-
-            soln_root = task_info["specification"].soln_path
-            soln_equivalent = os.path.join(soln_root, in_submission)
-            filename = os.path.split(soln_equivalent)[1]
+            soln_equivalent = context_utils.extract(
+                prev_context,
+                "ref_file_path"
+            )
+            ref_filename = context_utils.extract(
+                prev_context,
+                "ref_filename"
+            )
 
             if not os.path.isfile(soln_equivalent):
                 raise context_utils.ContextCreationError(
                     self,
-                    f"Target file {in_submission} does not exist in the"
+                    f"Target file {soln_equivalent} does not exist in the"
                     f" solution directory."
                 )
 
@@ -1083,7 +1107,7 @@ class SolnCodeContext(AutoContext):
                 contents = fin.read()
 
             try:
-                node = mast.parse(contents, filename=filename)
+                node = mast.parse(contents, filename=ref_filename)
             except Exception as e:
                 raise context_utils.ContextCreationError(
                     self,
@@ -1099,7 +1123,7 @@ class SolnCodeContext(AutoContext):
 
         # Figure out if we need to use automatic dependencies:
         if depends is None:
-            depends = auto("filename", "file_path")
+            depends = auto("ref_filename", "ref_file_path")
 
         # Now we can call the super constructor
         super().__init__(
@@ -1145,12 +1169,12 @@ class ModuleContext(AutoContext):
         )
     )
 
-    def display_result(context):
+    def display_result(self, context):
         """
         Context result display function which lists names defined in the
         loaded module.
         """
-        loaded = context["module"]
+        loaded = context[self._to]
         defined = [
             name
             for name in dir(loaded)
@@ -1233,7 +1257,7 @@ class ModuleContext(AutoContext):
         super().__init__(
             description=self._description,
             builder=establish_context,
-            display_product=ModuleContext.display_result,
+            display_product=self.display_result,
             depends=depends,
             hidden=hidden
         )
@@ -1414,6 +1438,10 @@ class DocstringsContext(AutoContext):
 
             return { "docstrings": docsmap }
 
+        # Figure out if we need to use automatic dependencies:
+        if depends is None:
+            depends = auto("defs", "module")
+
         super().__init__(
             description=(
                 "The function docstrings",
@@ -1680,14 +1708,6 @@ random chance.):
     return display_distribution_results
 
 
-# Register a factory for a default SolnModuleContext as an on-demand
-# option for the "ref_module" slot.
-AutoContext.on_demand(
-    (lambda: ModuleContext()),
-    "ref_module"
-)
-
-
 class SiftedContext(Context):
     """
     Working from the "output" and "ref_output" slots (or some other
