@@ -1,9 +1,9 @@
 """
-harness.py
-
 Tools for testing Python code and recording things like results, printed
 output, or even traces of calls to certain functions.
 
+harness.py
+
 These tools start with a few functions for creating "payloads" which are
 functions that take a context dictionary as a single argument and which
 return dictionaries of new context slots to establish (it's no
@@ -316,14 +316,21 @@ def create_run_harness_payload(
 # Harness augmentation functions #
 #--------------------------------#
 
-def run_for_base_and_ref_values(payload):
+def run_for_base_and_ref_values(payload, used_by_both=None):
     """
     Accepts a payload function and returns a modified payload function
     which runs the provided function twice, the second time using ref_*
-    values where available and setting ref_* versions of the original
-    payload's result slots. Note that when applying multiple payload
-    augmentations, this one should be applied last.
+    context values and setting ref_* versions of the original payload's
+    result slots. If a certain non-ref_* value needs to be available to
+    the reference payload other than the standard
+    `potluck.context_utils.BASE_CONTEXT_SLOTS`, it must be provided in
+    the "used_by_both" list.
+
+    Note that when applying multiple payload augmentations, this one
+    should be applied last.
     """
+    used_by_both = used_by_both or []
+
     def double_payload(context):
         """
         Runs a payload twice, once normally and again against a context
@@ -337,13 +344,22 @@ def run_for_base_and_ref_values(payload):
 
         # Create a context where each ref_* slot value is assigned to
         # the equivalent non-ref_* slot
-        collapsed_context = copy.copy(context)
-        for key in collapsed_context:
-            if key.startswith("ref_"):
-                collapsed_context[key[4:]] = collapsed_context[key]
+        ref_context = {
+            key: context[key]
+            for key in context_utils.BASE_CONTEXT_SLOTS
+        }
+        for key in context:
+            if key in used_by_both:
+                ref_context[key] = context[key]
+            elif key.startswith("ref_"):
+                ref_context[key[4:]] = context[key]
 
         # Get results from collapsed context
-        ref_result = payload(collapsed_context)
+        try:
+            ref_result = payload(ref_context)
+        except context_utils.MissingContextError as e:
+            e.args = (e.args[0] + " (in reference payload)", ) + e.args[1:]
+            raise e
 
         # Assign collapsed context results into final result under ref_*
         # versions of their slots
@@ -424,8 +440,9 @@ def capturing_printed_output(payload, capture_errors=False):
         try:
             result = payload(context)
         except Exception:
+            # TODO: This is wrong and bad..?
+            result = {}
             if capture_errors:
-                # TODO: path reworking here, if not in html_tools!
                 string_stdout.write('\n' + html_tools.string_traceback())
             else:
                 raise
@@ -684,7 +701,7 @@ def tracing_function_calls(payload, trace_targets, state_function):
 
                 # Grab arguments if we can:
                 if not event.startswith("c_"):
-                    info["args"] = frame.f_locals
+                    info["args"] = copy.copy(frame.f_locals)
 
                 # Push this info object onto the appropriate stack
                 if fname not in trace_stacks:
@@ -755,7 +772,7 @@ def walk_trace(trace):
     """
     for entry in trace:
         yield entry
-        walk_trace(entry["during"])
+        yield from walk_trace(entry["during"])
 
 
 def sampling_distribution_of_results(
@@ -1012,11 +1029,13 @@ def set_dot_attr(obj, dot_attr, value):
 
 def warp_turtle(context):
     """
-    Disables turtle tracing. Use as a setup function with `with_setup`
-    and/or via `specifications.HasPayload.do_setup`. Note that you MUST
-    also use `finalize_drawing` as a cleanup function, or else some
-    elements may not actually get drawn.
+    Disables turtle tracing, and resets turtle state. Use as a setup
+    function with `with_setup` and/or via
+    `specifications.HasPayload.do_setup`. Note that you MUST also use
+    `finalize_drawing` as a cleanup function, or else some elements may
+    not actually get drawn.
     """
+    turtle.reset()
     turtle.tracer(0, 0)
     return context
 
