@@ -1,9 +1,11 @@
+# -*- coding: utf-8 -*-
 """
 Tools for building HTML strings.
 
 html_tools.py
 """
 
+
 import traceback
 import difflib
 import html
@@ -52,10 +54,10 @@ def build_list(items, ordered=False):
     """
     tag = "ol" if ordered else "ul"
     return (
-        f"<{tag}>"
-      + "\n".join(f"<li>{item}</li>" for item in items)
-      + f"\n</{tag}>"
-    )
+        "<{tag}>"
+      + "\n".join("<li>{}</li>".format(item) for item in items)
+      + "\n</{tag}>"
+    ).format(tag=tag)
 
 
 def build_html_details(title, content, classes=None):
@@ -83,29 +85,35 @@ def build_html_tabs(tabs):
     """
     tab_pieces = [
         (
-            f"""
+            """
 <li
- class='tab{" selected" if i == 0 else ""}' onclick='toggleTab();'
+ class='tab{selected}' onclick='toggleTab();'
  tabindex="0"
  aria-label="Activate to display {title} after this list."
 >
 {title}
 </li>
-""",
-            f"""
+"""         .format(
+                selected=" selected" if i == 0 else "",
+                title=title
+            ),
+            """
 <section
- class='tab-content{" selected" if i == 0 else ""}'
+ class='tab-content{selected}'
  tabindex="0"
 >
 {content}
 </section>
-"""
+"""         .format(
+                selected=" selected" if i == 0 else "",
+                content=content
+            )
         )
         for i, (title, content) in enumerate(tabs)
     ]
     tabs = '\n\n'.join([piece[0] for piece in tab_pieces])
     contents = '\n\n'.join([piece[1] for piece in tab_pieces])
-    return f"""
+    return """
 <div class="tabs" role="presentation">
   <ul class="tabs-top">
 {tabs}
@@ -114,7 +122,10 @@ def build_html_tabs(tabs):
 {contents}
   </div>
 </div>
-"""
+""" .format(
+        tabs=tabs,
+        contents=contents
+    )
 
 
 def fileslug(filename):
@@ -132,7 +143,11 @@ def line_id(taskid, filename, lineno):
     Generates an HTML ID for a code line, given the task ID, file name,
     and line number.
     """
-    return f"{taskid}_{fileslug(filename)}_codeline_{lineno}"
+    return "{taskid}_{slug}_codeline_{lineno}".format(
+        taskid=taskid,
+        slug=fileslug(filename),
+        lineno=lineno
+    )
 
 
 def block_id(taskid, filename):
@@ -140,7 +155,10 @@ def block_id(taskid, filename):
     Generates an HTML ID for a code block, given the task ID and file
     name.
     """
-    return f"{taskid}_code_{fileslug(filename)}"
+    return "{taskid}_code_{slug}".format(
+        taskid=taskid,
+        slug=fileslug(filename)
+    )
 
 
 def html_link_to_line(taskid, filename, lineno):
@@ -150,10 +168,15 @@ def html_link_to_line(taskid, filename, lineno):
     """
     blockid = block_id(taskid, filename)
     lineid = line_id(taskid, filename, lineno)
-    focus_call = f"focusCodeLine('{blockid}', '{lineid}'); return true;"
+    focus_call = "focusCodeLine('{blockid}', '{lineid}'); return true;"\
+        .format(blockid=blockid, lineid=lineid)
     return (
-        f'<a class="lineref" href="#{lineid}"'
-        f' onclick="{focus_call}">{lineno}</a>'
+        '<a class="lineref" href="#{lineid}"'
+        ' onclick="{focus_call}">{lineno}</a>'
+    ).format(
+        lineid=lineid,
+        focus_call=focus_call,
+        lineno=lineno
     )
 
 
@@ -289,7 +312,7 @@ def big_repr(thing):
             )
             return '{\n' + indent(stuff, 2) + '\n}'
         elif type(thing) is str and '\n' in thing:
-            return f"'''\\\n{thing}'''"
+            return "'''\\\n{thing}'''".format(thing=thing)
         # else we fall out and return base
 
     return base
@@ -321,7 +344,9 @@ def build_display_box(text):
     """
     # TODO: A disabled textarea is such a hack! Instead, use an inner pre
     # inside an outer div with CSS to make it scrollable.
-    return f'<textarea class="display_box" disabled>{text}</textarea>'
+    return '<textarea class="display_box" disabled>{text}</textarea>'.format(
+        text=text
+    )
 
 
 def dynamic_html_repr(thing, reasonable=200, limit=10000):
@@ -337,7 +362,7 @@ def dynamic_html_repr(thing, reasonable=200, limit=10000):
     """
     rep = big_repr(thing)
     if len(rep) <= reasonable:
-        return f'<pre class="short_repr">{rep}</pre>'
+        return '<pre class="short_repr">{rep}</pre>'.format(rep=rep)
     else:
         return build_display_box(truncate(rep, limit))
 
@@ -396,7 +421,10 @@ def rewrite_traceback_filenames(raw_traceback, prefix_map):
     result = raw_traceback
     for prefix in prefix_map:
         replace = prefix_map[prefix]
-        result = result.replace(f'File "{prefix}', f'File "{replace}')
+        result = result.replace(
+            'File "{prefix}'.format(prefix=prefix),
+            'File "{replace}'.format(replace=replace)
+        )
 
     return result
 
