@@ -96,7 +96,7 @@ def get_css():
     Returns the CSS code for reports as a string.
     """
     if RESOURCES_DIR is None:
-        pkg_files = importlib_resources.files("potluck.resources")
+        pkg_files = importlib_resources.files("potluck") / "resources"
         return pkg_files.joinpath("potluck.css").read_text()
     else:
         with open(os.path.join(RESOURCES_DIR, "potluck.css")) as fin:
@@ -108,7 +108,7 @@ def get_js():
     Returns the Javascript code for reports as a string.
     """
     if RESOURCES_DIR is None:
-        pkg_files = importlib_resources.files("potluck.resources")
+        pkg_files = importlib_resources.files("potluck") / "resources"
         return pkg_files.joinpath("potluck.js").read_text()
     else:
         with open(os.path.join(RESOURCES_DIR, "potluck.js")) as fin:
@@ -124,19 +124,30 @@ def load_template(template_name):
         return J2_ENV.get_template(template_name)
     except jinja2.exceptions.TemplateSyntaxError as e:
         logging.log(
-            f"Syntax error while building Jinja2 template:"
-            f" '{template_name}'\n"
-            f"Error occurred on line: {e.lineno}\n"
-            f"Error was: {str(e)}\n"
-            '-' * 80
+            (
+                "Syntax error while building Jinja2 template:"
+                " '{template_name}'\n"
+                "Error occurred on line: {lineno}\n"
+                "Error was: {error}\n"
+                '-' * 80
+            ).format(
+                template_name=template_name,
+                lineno=e.lineno,
+                error=str(e)
+            )
         )
         raise
     except Exception as e:
         logging.log(
-            f"Unexpected non-syntax error while building Jinja2"
-            f" template: '{template_name}'\n"
-            f"Error was: {e}\n"
-            '-' * 80
+            (
+                "Unexpected non-syntax error while building Jinja2"
+                " template: '{template_name}'\n"
+                "Error was: {error}\n"
+                '-' * 80
+            ).format(
+                template_name=template_name,
+                error=str(e)
+            )
         )
         raise
 
@@ -258,7 +269,11 @@ def render_code(taskid, filename, raw_code):
         annotate_line(taskid, filename, n + 1, line)
         for n, line in enumerate(highlighted.split('\n'))
     )
-    return f'<pre id="{taskid}_code_{filename}">' + annotated + '</pre>'
+    return (
+        '<pre id="{}">'.format(html_tools.block_id(taskid, filename))
+      + annotated
+      + '</pre>'
+    )
 
 
 def annotate_line(taskid, filename, lineno, line_html):
@@ -274,13 +289,18 @@ def annotate_line(taskid, filename, lineno, line_html):
     lineid = html_tools.line_id(taskid, filename, lineno)
     block_id = html_tools.block_id(taskid, filename)
     return (
-        f'<span id="{lineid}"'
-        f' class="codeline codeline_plain"'
-        f' onclick="focusCodeLine(\'{block_id}\', \'{lineid}\');'
-        f' return false">'
-        f'<span class="linenum">{lineno} </span>'
-        f' {line_html}'
-        f'</span>'
+        '<span id="{lineid}"'
+        ' class="codeline codeline_plain"'
+        ' onclick="focusCodeLine(\'{block_id}\', \'{lineid}\');'
+        ' return false">'
+        '<span class="linenum">{lineno} </span>'
+        ' {line_html}'
+        '</span>'
+    ).format(
+        lineid=lineid,
+        block_id=block_id,
+        lineno=lineno,
+        line_html=line_html
     )
 
 
@@ -389,7 +409,7 @@ def render_rubric_row(row, blank=False):
         "rubric_row",
         html_tools.status_css_class(row["status"])
     ] + [
-        f"tag-{k}-{v}"
+        "tag-{k}-{v}".format(k=k, v=v)
         for (k, v) in row.get("tags", {}).items()
     ]
     return '<div class="{}">{}</div>'.format(
