@@ -8,7 +8,11 @@
  */
 
 // jshint esversion: 6
-/* global window, document, console */
+/* global window, document, console, IntersectionObserver */
+/* jshint -W097 */
+/* jshint -W014 */
+/* jshint -W080 */
+/* jshint -W040 */
 
 "use strict";
 
@@ -178,12 +182,12 @@ function validate_task_submit(task_form) {
 
     if (current_filename != task_filename) {
         if (unsubmitted) {
-            messages["filename"] = (
+            messages.filename = (
                 "You must select a file named <code>" + task_filename
               + "</code>"
             );
         } else {
-            messages["filename"] = (
+            messages.filename = (
                 "To resubmit, select a file named <code>" + task_filename
               + "</code>"
             );
@@ -195,12 +199,12 @@ function validate_task_submit(task_form) {
 
     if (spent == "") {
         if (unsubmitted) {
-            messages["time_spent"] = "You must estimate your time spent.";
+            messages.time_spent = "You must estimate your time spent.";
         } else {
-            messages["time_spent"] = "To resubmit, estimate your time spent.";
+            messages.time_spent = "To resubmit, estimate your time spent.";
         }
     } else if (isNaN(parseFloat(spent))) {
-        messages["time_spent"] = (
+        messages.time_spent = (
             "Please enter a decimal number of hours (like '0.3', '2', or"
           + " '3.5') for time spent."
         );
@@ -274,7 +278,7 @@ for (let task_form of document.querySelectorAll("form.task_submit")) {
     }
 }
 
-let validation_switch = document.getElementById("enable_validation")
+let validation_switch = document.getElementById("enable_validation");
 if (validation_switch != null) {
     validation_switch.addEventListener(
         "change",
@@ -294,7 +298,7 @@ if (validation_switch != null) {
  * Feedback interaction *
  *----------------------*/
 
-let hide_accomplished = document.getElementById("hide_accomplished_goals")
+let hide_accomplished = document.getElementById("hide_accomplished_goals");
 if (hide_accomplished != null) {
     hide_accomplished.addEventListener(
         "change",
@@ -421,7 +425,7 @@ function stop_floating(bin) {
     bin.style.removeProperty("width");
     bin.style.removeProperty("height");
 
-    let label = bin.querySelector(".bin_label")
+    let label = bin.querySelector(".bin_label");
     label.style.removeProperty("position");
     label.style.removeProperty("top");
     label.style.removeProperty("left");
@@ -608,7 +612,10 @@ for (let context of document.querySelectorAll(".sticky_context")) {
 function sticky_bin_ancestor(element) {
     if (!element.parentNode) {
         return null;
-    } else if (element.parentNode.classList.contains("sticky_bin")) {
+    } else if (
+        element.parentNode.classList
+     && element.parentNode.classList.contains("sticky_bin")
+   ) {
         return element.parentNode;
     } else {
         return sticky_bin_ancestor(element.parentNode);
