@@ -14,13 +14,13 @@ are JSON-in, JSON-out, but raw strings can be requested instead.
 """
 
 from flask import json
-import os, sys, copy, time, datetime
+import os, sys, copy, time
 import threading
 import multiprocessing, multiprocessing.connection, multiprocessing.managers
 
 # Python 2/3 dual compatibility
 if sys.version_info[0] < 3:
-    reload(sys)
+    reload(sys) # noqa F821
     sys.setdefaultencoding('utf-8')
     import socket
     ConnectionRefusedError = socket.error
@@ -33,8 +33,10 @@ else:
     AuthenticationError = multiprocessing.context.AuthenticationError
 
 # Python 2.7.5 multiprocessing bug workaround
-# Bug thread of same behavior: https://bugzilla.redhat.com/show_bug.cgi?id=1717330
-# Python bugfix: https://github.com/python/cpython/commit/e8a57b98ec8f2b161d4ad68ecc1433c9e3caad57
+# Bug thread of same behavior:
+# https://bugzilla.redhat.com/show_bug.cgi?id=1717330
+# Python bugfix:
+# https://github.com/python/cpython/commit/e8a57b98ec8f2b161d4ad68ecc1433c9e3caad57
 # Source for this workaround: https://github.com/oamg/leapp/pull/533/files
 
 # Implements:
@@ -60,6 +62,7 @@ else:
 
     setattr(multiprocessing.util, 'Finalize', FixedFinalize)
 
+
 #----------------------#
 # Core caching routine #
 #----------------------#
@@ -72,6 +75,7 @@ class AbortGeneration:
     def __init__(self, replacement):
         self.replacement = replacement
 
+
 class NotInCache:
     """
     Placeholder for recognizing that a value is not in the cache (when
@@ -79,6 +83,7 @@ class NotInCache:
     """
     pass
 
+
 def _gen_or_get_cached(lock, cache, cache_key, check_dirty, result_generator):
     """
     Common functionality that uses a reentrant lock and a cache
@@ -189,6 +194,7 @@ class View:
         """
         raise NotImplementedError("Don't use the base View class.")
 
+
 class AsIs(View):
     """
     A pass-through view that returns strings unaltered.
@@ -203,6 +209,7 @@ class AsIs(View):
         """Returns the string it is given unaltered."""
         return string
 
+
 class AsJSON(View):
     """
     A view that converts objects to JSON for file storage and back on
@@ -219,10 +226,11 @@ class AsJSON(View):
         Returns a JSON object parsed from the string.
         Returns None if it gets None.
         """
-        if string == None:
+        if string is None:
             return None
         return json.loads(string)
 
+
 def build_view(name, encoder, decoder, pass_none=True):
     """
     Function for building a view given a name, an encoding function, and
@@ -241,7 +249,7 @@ def build_view(name, encoder, decoder, pass_none=True):
 
         @staticmethod
         def decode(string):
-            if pass_none and string == None:
+            if pass_none and string is None:
                 return None
             return decoder(string)
 
@@ -254,6 +262,7 @@ def build_view(name, encoder, decoder, pass_none=True):
 
     return SyntheticView
 
+
 #--------------------------#
 # Process-local singletons #
 #--------------------------#
@@ -269,10 +278,11 @@ def build_or_get_singleton(cache_key, builder):
         _CACHE_LOCK,
         _CACHE,
         cache_key,
-        lambda key, ts: ts != None,
+        lambda key, ts: ts is not None,
         builder
     )
 
+
 #--------------------------------------#
 # Process-local read-only file caching #
 #--------------------------------------#
@@ -300,10 +310,11 @@ def build_file_freshness_checker(missing=Exception):
 
         # File is changed if the mtime is after the given cache
         # timestamp, or if the timestamp is None
-        return ts == None or mtime >= ts
+        return ts is None or mtime >= ts
 
     return check_file_is_changed
 
+
 def build_file_reader(view=AsJSON):
     """
     Builds a file reader function which returns the result of the given
@@ -342,6 +353,8 @@ def build_file_reader(view=AsJSON):
 # size, and maintain staleness so we can get rid of stale items...
 _CACHE_LOCK = threading.RLock() # Make this reentrant just in case...
 _CACHE = {}
+
+
 def load_or_get_cached(
     filename,
     view=AsJSON,
@@ -364,7 +377,7 @@ def load_or_get_cached(
     """
 
     # Figure out our view object (& cache key):
-    if view == None:
+    if view is None:
         view = AsIs
 
     cache_key = cache_key_for(filename, view)
@@ -393,6 +406,7 @@ _ACCESS_CACHE = {}
 # The _FileAccess proxy to use for file access:
 _ACCESS = None
 
+
 def read_file(
     filename,
     view=AsJSON,
@@ -436,7 +450,7 @@ def read_file(
     """
 
     # Figure out our view function (& cache key just in case):
-    if view == None:
+    if view is None:
         view = AsIs
 
     # Define our reader function which incorporates our view:
@@ -481,7 +495,7 @@ def write_file(filename, content, view=AsJSON, cache=True):
     encoded string, will be cached. The view may be given as None to
     write the content directly to the file as a string.
     """
-    if view == None:
+    if view is None:
         view = AsIs
 
     file_content = view.encode(content)
@@ -551,28 +565,37 @@ class FileSystemManager(multiprocessing.managers.BaseManager):
     """
     pass
 
+
 FileSystemManager.register("get", _FileAccessor)
 
 
 # The manager connection for this process:
 _MANAGER = None
+
+
 def init_sync(port=51723, key=None):
     """
     init_sync should be called once per process, ideally early in the
     life of the process, like right after importing the sync module.
     Calling access_file before init_sync will fail. A file named
-    'syncauth' must exist unless a key is given (should be a
-    byte-string). Don't rely too heavily on the default port, since any
-    two apps using sync with the default port on the same machine will
-    collide.
+    'syncauth' should exist unless a key is given (should be a
+    byte-string). If 'syncauth' doesn't exist, it will be created. Don't
+    rely too heavily on the default port, since any two apps using sync
+    with the default port on the same machine will collide.
     """
     global _MANAGER, _ACCESS
     # Get synchronization key:
-    if key == None:
+    if key is None:
         try:
-            with open('syncauth', 'rb') as fin:
-                key = fin.read()
-        except:
+            if os.path.exists('syncauth'):
+                with open('syncauth', 'rb') as fin:
+                    key = fin.read()
+            else:
+                print("Creating new sync secret file 'syncauth'.")
+                key = os.urandom(16)
+                with open('syncauth', 'wb') as fout:
+                    fout.write(key)
+        except Exception:
             raise IOError_or_FileNotFoundError(
                 "Unable to access 'syncauth' file.\n"
               + "  Create it and put some random bytes in there, or pass"
@@ -586,10 +609,12 @@ def init_sync(port=51723, key=None):
     while True:
         print("Attempting to connect to MP manager...")
 
-        # Reduce multiprocessing's connection timeout value since we're pretty
-        # sure we'll fail first and succeed later
+        # Reduce multiprocessing's connection timeout value since we're
+        # pretty sure we'll fail first and succeed later
         old_init_timeout = multiprocessing.connection._init_timeout
-        multiprocessing.connection._init_timeout = lambda t=2.0: time.time() + t
+        multiprocessing.connection._init_timeout = lambda t=2.0: (
+            time.time() + t
+        )
         try:
             # NOTE: This call may just never return if we accidentally
             # connect to e.g., an HTTP server.
